import java.io.*;

import antlr.CommonAST;
import antlr.collections.AST;

class Main {

    static int count = 0;


    static boolean showTree = false;
    public static void main(String[] args) {
        // Use a try/catch block for parser exceptions
        try {
            // if we have at least one command-line argument
            if (args.length > 0 ) {
                System.err.println("Parsing...");

                // for each directory/file specified on the command line
                for(int i=0; i< args.length;i++) {
                    if ( args[i].equals("-showtree") ) {
                        showTree = true;
                    }
                    else {
                        doFile(new File(args[i])); // parse it
                    }
                } }
            else
                System.err.println("Usage: java Main [-showtree] "+
                                   "<directory or file name>");
        }
        catch(Exception e) {
            System.err.println("exception: "+e);
            e.printStackTrace(System.err);   // so we can get stack trace
        }
    }


    // This method decides what action to take based on the type of
    //   file we are looking at
    public static void doFile(File f)
        throws Exception {
        // If this is a directory, walk each file/dir in that directory
        if (f.isDirectory()) {
            String files[] = f.list();
            for(int i=0; i < files.length; i++)
                doFile(new File(f, files[i]));
        }

        // otherwise, if this is a java file, parse it!
        else if ((f.getName().length()>4) &&
                 f.getName().substring(f.getName().length()-4).equals(".lfp")) {
            System.err.println("   "+f.getAbsolutePath());
            // parseFile(f.getName(), new FileInputStream(f));
            parseFile(f.getName(), new BufferedReader(new FileReader(f)));
        }
    }

    // Here's where we do the real work...
    public static void parseFile(String f, Reader r)
        throws Exception {
        try {
            // Create a scanner that reads from the input stream passed to us
            LfP_Lexer lexer = new LfP_Lexer(r);
            lexer.setFilename(f);

            // Create a parser that reads from the scanner
            LfP_Parser parser = new LfP_Parser(lexer);
            parser.setFilename(f);

            System.err.println("--> parsing starts");
            parser.lfp_model();
            System.err.println("<-- parsing ends");
            System.err.println(" Main.count : " + count);
            CommonAST theAST=(CommonAST)parser.getAST();
            java.io.File fic=new java.io.File("out.xml");
            java.io.FileWriter writer=new java.io.FileWriter(fic);
            writer.write("<root>");
            theAST.xmlSerialize(writer);
            writer.write("\n</root>");
            writer.flush();
        }
        catch (Exception e) {
            System.err.println("parser exception: "+e);
            e.printStackTrace();   // so we can get stack trace		
        }
    }
	
}

