/* Calculate an expression read from stdin or from the arguments passed to
 * the program (using stringstream's)
 */
#include <iostream>
#include <fstream>
#include <algorithm>
#include "antlr/AST.hpp"
#include "LfP_Lexer.hpp"
#include "LfP_Parser.hpp"

std::string xmltransf(const std::string in)
{
	std::string out(in);
	int pos=out.find("<");
	while(pos!=-1)
	{
		if(pos>0)
			out=out.substr(0,pos-1)+std::string("&lt;")+out.substr(pos+1);
		else
			out=std::string("&lt;")+out.substr(1);
		pos=out.find("<");
	}
	return out;
}

void printxml(antlr::RefAST t)
{
	std::string node(t->typeName());
	std::cout << "<" << node << " type=\"" << 
		t->getType() << "\" text=\""<< xmltransf(t->getText()) << "\">\n";
	antlr::RefAST ch=t->getFirstChild();
	for(unsigned int i=0;i<t->getNumberOfChildren();++i)
	{
		printxml(ch);
		ch=ch->getNextSibling();
	}
	std::cout << "</" << node << ">\n";
}

int main( int argc, char* argv[] )
{
	ANTLR_USING_NAMESPACE(std)
	ANTLR_USING_NAMESPACE(antlr)
	try
	{
		if( argc != 2 )
		{
			std::cerr << "usage CppLfp filename\n";
			return 0;
		}

		std::ifstream input(argv[1]);

		LfP_Lexer lexer(input);
		lexer.setFilename(argv[1]);

		LfP_Parser parser(lexer);
		parser.setFilename(argv[1]);

		ASTFactory ast_factory;
		parser.initializeASTFactory(ast_factory);
		parser.setASTFactory(&ast_factory);

		// Parse the input expression
		parser.lfp_model();
		RefAST t = parser.getAST();
		if( t )
		{
			cout << "<root>\n";
			while(t)
			{
				printxml(t);
				t=t->getNextSibling();
			}
			cout << "</root>\n";
		}
		else
			cout << "No tree produced" << endl;

	}
	catch(ANTLRException& e)
	{
		cerr << "Parse exception: " << e.toString() << endl;
		return -1;
	}
	catch(exception& e)
	{
		cerr << "exception: " << e.what() << endl;
		return -1;
	}
	return 0;
}
